(theory FiniteMap

 :written_by "Philipp Ruemmer, Cesare Tinelli" 
 :date "10/07/2009"

 :note "Formalization developed in cooperation with  
        Daniel Kroening, Georg Weissenbacher, and Ofer Strichman"

 :sorts ((Set 1) (Map 2))

 :funs ((par (X Y) (emptyMap (Map X Y))) 
        (par (X Y) (apply (Map X Y) X Y))
        (par (X Y) (overwrite (Map X Y) X Y (Map X Y)))

        (par (X Y) (domain (Map X Y) (Set X)))
        (par (X Y) (range (Map X Y) (Set Y)))
        (par (X Y) (restrict (Map X Y) (Set X) (Map X Y)))
        (par (X Y) (subtract (Map X Y) (Set X) (Map X Y)))
	)

 :funs ((par (X) (subset (Set X) (Set X) Bool :chainable))
        (par (X) (in X (Set X) Bool))
       )

 :definition
 "Let Q be a set of sort symbols including Set and Bool, and let S
  be the set of all (ground) sort terms over Q. For any s in S
  and any function symbol f, let [[s]] and [[f]] respectively denote
  the interpretation of s and f in some given structure.

  For any S like the above, FiniteMaps(S) is the theory consisting of all
  structures satisfying the following restrictions for all s, s' in S:

  - [[(Set s)]] is the set of all finite subsets of [[s]] 

  - [[f]] is as expected if f is in {subset, in}.

  - [[(Map s s')]] is the set of all finite partial maps from [[s]] to [[s']] 

  - [[emptyMap]] is the function from [[s]] to [[s']] undefined everywhere.

  - For all finite maps m from [[s]] to [[s']] and x in [[s]], 

      [[apply]](m, x) = m(x) if m is defined on x (and is arbitrary otherwise)

  - For all finite maps m from [[s]] to [[s']], a, x in [[s]], and b in [[s']],

                                  b           if x = a
      [[overwrite]](m, a, b)(x) = m(x)        if x != a and m is defined on x
                                  undefined   otherwise 

  - For all finite maps m from [[s]] to [[s']], 

      [[domain]](m) = { x in [[s]] | m is defined on x }

  - For all finite maps m from [[s]] to [[s']],

      [[range]](m) = { y in [[s']] | there is an x in [[s]] s.t. m(x) = y }

  - For all finite maps m from [[s]] to [[s']], finite subsets r of [[s]],
    and x in [[s]],

                              m(x)        if m is defined on x and x is in r
      [[restrict]](m, r)(x) = 
                              undefined   otherwise

  - For all finite maps m from [[s]] to [[s']], finite subsets r of [[s]],
    and x in [[s]],

                              m(x)        if m is defined on x and x is not in r
      [[subtract]](m, r)(x) = 
                              undefined   otherwise

 "

 :note 
 "For any given S, FiniteMaps(S) has several models which differ not
  only on how they interpret the base sorts of S other than Int and Bool,
  but also on how they interpret the 'apply' function symbol when the 
  value of a map is queried at a point outside of its domain.
  "
 )

