(theory FiniteRelationWithCard

 :written_by "Philipp Ruemmer, Cesare Tinelli" 
 :date "10/07/2009"

 :note "Formalization developed in cooperation with  
        Daniel Kroening, Georg Weissenbacher, and Ofer Strichman"

 :sorts ((Set 1) (Int 0))

 :sorts_description
 "All n-ary sort symbols of the form (ind Tuple n),
   where n is a numeral > 0.
 "	
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind tuple n) Y_1 ... Y_n ((ind Tuple n) Y_1 ... Y_n)))
  where n is a numeral > 0.
 "
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind project n i) ((ind Tuple n) Y_1 ... Y_n) Y_i))
  where n is a numeral > 0 and i is a numeral in [1 .. n].
 "
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind product n) (Set Y_1) ... (Set Y_n)
                     (Set ((ind Tuple n) Y_1 ... Y_n))))
  where n is a numeral > 0.
 "

 :funs ((par (X) (emptySet (Set X)))
        (par (X) (insert X (Set X) (Set X) :right_associative))
        (par (X) (in X (Set X) Bool))
        (par (X) (subset (Set X) (Set X) Bool :chainable))
        (par (X) (union (Set X) (Set X) (Set X) :left_associative))
        (par (X) (inter (Set X) (Set X) (Set X) :left_associative))
        (par (X) (setminus (Set X) (Set X) (Set X) :left_associative))
	)

 :funs ((par (X) (card (Set X) Int))
        )

 :funs ((NUMERAL Int) 
        (- Int Int)     ; unary minus
        (- Int Int Int :left_associative) ; binary minus
        (+ Int Int Int :left_associative) 
        (* Int Int Int :left_associative)
        (<= Int Int Bool :chainable)
        (< Int Int Bool :chainable)
        (>= Int Int Bool :chainable)
        (> Int Int Bool :chainable)
        )

 :definition 
 "Let Q be a set of sort symbols including Set, Int, Bool, and (Tuple
  n) for each n > 0, and let S be the set of all (ground) sort terms
  over Q.  For any s in S and any function symbol f, let [[s]] and
  [[f]] respectively denote the interpretation of s and f in some
  given structure.

  For any S, FiniteRelationsWithCard(S) is the theory consisting of
  all structures satisfying the following restrictions for all 
  s, s1, ..., sn in S:

  - [[Int]] is the set of the set Z of integer numbers.

  - [[f]] is as expected if f is in {-,+,*,<=,<,>,>=}.

  - [[n]] is as expected if n is a numeral.

  - [[(Set s)]] is the set of all finite subsets of [[s]].

  - [[((ind Tuple n) s1 ... sn)]] is the set [[s1]] x ... x [[sn]]
    of all n-tuples over [[s1]], ..., [[sn]].

  - For all n > 0, x1 in [[s1]], ..., xn in [[sn]],

      [[(ind tuple n)]](x1, ..., xn) = (x1, ..., xn)

  - For all n > 0, 0 < i <= n, and tuples (x1, ..., xn) in
    [[s1]] x ... x [[sn]],

      [[(ind project n i)]]((x1, ..., xn)) = xi

  - For all n > 0, and finite subsets b1 of [[b1], ..., mn of [[bn],

      [[(ind product n)]](b1, ..., bn) =
        { (x1, ..., xn) | xi in bi for all i = 1, ..., n }

  - For all finite subsets b of [[s]] and k >= 0,

      [[(card b)]] = k  iff  b has k elements 
 "

 )

