(theory LRM

 :written_by "Philipp Ruemmer, Cesare Tinelli" 
 :date "10/07/2009"

 :note "Formalization developed in cooperation with  
        Daniel Kroening, Georg Weissenbacher, and Ofer Strichman"


;-------------------------
; from FiniteSetsWithCard
;------------------------- 

 :sorts ((Set 1) (Int 0))

 :funs ((par (X) (emptySet (Set X)))
        (par (X) (insert X (Set X) (Set X) :right_associative))
        (par (X) (in X (Set X) Bool))
        (par (X) (subset (Set X) (Set X) Bool :chainable))
        (par (X) (union (Set X) (Set X) (Set X) :left_associative))
        (par (X) (inter (Set X) (Set X) (Set X) :left_associative))
        (par (X) (setminus (Set X) (Set X) (Set X) :left_associative))
	)

 :funs ((par (X) (card (Set X) Int))
        )

 :funs ((NUMERAL Int) 
        (- Int Int)     ; unary minus
        (- Int Int Int :left_associative) ; binary minus
        (+ Int Int Int :left_associative) 
        (* Int Int Int :left_associative)
        (<= Int Int Bool :chainable)
        (< Int Int Bool :chainable)
        (>= Int IntBool :chainable)
        (> Int Int Bool :chainable)
        )


;------------------------------
; from FiniteRelationsWithCard
;------------------------------ 

;; :sorts ((Set 1) (Int 0))

 :sorts_description
 "All n-ary sort symbols of the form (ind Tuple n),
   where n is a numeral > 0.
 "	
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind tuple n) Y_1 ... Y_n ((ind Tuple n) Y_1 ... Y_n)))
  where n is a numeral > 0.
 "
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind project n i) ((ind Tuple n) Y_1 ... Y_n) Y_i))
  where n is a numeral > 0 and i is a numeral in [1 .. n].
 "
 :funs_description
 "All functions symbols with declaration of the form
  (par (Y_1 ... Y_n)
    ((ind product n) (Set Y_1) ... (Set Y_n)
                     (Set ((ind Tuple n) Y_1 ... Y_n))))
  where n is a numeral > 0.
 "

;; :funs ((par (X) (emptySet (Set X)))
;;        (par (X) (insert X (Set X) (Set X) :right_associative))
;;        (par (X) (in X (Set X) Bool))
;;        (par (X) (subset (Set X) (Set X) Bool :chainable))
;;        (par (X) (union (Set X) (Set X) (Set X) :left_associative))
;;        (par (X) (inter (Set X) (Set X) (Set X) :left_associative))
;;        (par (X) (setminus (Set X) (Set X) (Set X) :left_associative))
;;	)
;;
;; :funs ((par (X) (card (Set X) Int))
;;        )
;;
;; :funs ((NUMERAL Int) 
;;        (- Int Int)     ; unary minus
;;        (- Int Int Int :left_associative) ; binary minus
;;        (+ Int Int Int :left_associative) 
;;        (* Int Int Int :left_associative)
;;        (<= Int Int Bool :chainable)
;;        (< Int Int Bool :chainable)
;;        (>= Int Int Bool :chainable)
;;        (> Int Int Bool :chainable)
;;        )


;----------------------
; From ListsWithLength
;----------------------

 :sorts ((List 1)
;;         (Int 0)
	)

 :funs ((par (X) (nil (List X)))
        (par (X) (cons X (List X) (List X) :right_associative))
        (par (X) (head (List X) X))
        (par (X) (tail (List X) (List X)))
        (par (X) (append (List X) (List X) (List X) :left_associative))
	)

 :funs ((par (X) (length (List X) Int))
        (par (X) (nth Int (List X) X))
        )

;; :funs ((NUMERAL Int) 
;;        (- Int Int)                       ; unary minus
;;        (- Int Int Int :left_associative) ; binary minus
;;        (+ Int Int Int :left_associative) 
;;        (* Int Int Int :left_associative)
;;        (<= Int Int Bool :chainable)
;;        (< Int Int Bool :chainable)
;;        (>= Int IntBool :chainable)
;;        (> Int Int Bool :chainable)
;;        )


;------------
; FiniteMaps
;------------

 :sorts ( ;(Set 1)
         (Map 2)
        )

 :funs ((par (X Y) (emptyMap (Map X Y))) 
        (par (X Y) (apply (Map X Y) X Y))
        (par (X Y) (overwrite (Map X Y) X Y (Map X Y)))

        (par (X Y) (domain (Map X Y) (Set X)))
        (par (X Y) (range (Map X Y) (Set Y)))
        (par (X Y) (restrict (Map X Y) (Set X) (Map X Y)))
        (par (X Y) (subtract (Map X Y) (Set X) (Map X Y)))
	)

;; :funs ((par (X) (subset (Set X) (Set X) Bool :chainable))
;;        (par (X) (in X (Set X) Bool))
;;       )


;-----
; New
;-----

 :funs ((par (X) (inds (List X) (Set Int)))
        (par (X) (elems (List X) (Set X)))
       )


 :definition
 "Let Q be a set of sort symbols including Set, Map, List, Bool, and
  (Tuple n) for each n > 0, and let S be the set of all (ground) sort
  terms over Q. For any s in S and any function symbol f, let [[s]]
  and [[f]] respectively denote the interpretation of s and f in some
  given structure.

  For any S like the above, LRM(S) is the theory consisting of all
  structures satisfying the following restrictions for all s, s', s1,
  ..., sn in S:

  - All restrictions present in the theory FiniteSetWithCard(S)
    concerning the sorts Int, Bool, (Set s) and the symbols -, +, *,
    <=, <, >, >=, numerals n, emptySet, insert, in, subset, union,
    inter, setminus, card.

  - All restrictions present in the theory FiniteRelationsWithCard(S)
    concerning the sorts ((ind Tuple n) s1 ... sn) and the symbols
    (ind tuple n), (ind project n i), (ind product n), for all
    numerals n > 0 and i in [1, ..., n].

  - All restrictions present in the theory ListsWithLength(S)
    concerning the sorts (List s) and the symbols nil, cons, nth,
    head, length, tail, append.

  - All restrictions present in the theory FiniteMaps(S) concerning
    the sorts (Map s s') and the symbols emptyMap, apply, overwrite,
    domain, range, restrict, subtract.

  - For all lists l and integers n with n = [[length]](l),
    [[inds]](l) = {1, ..., n}

  - For all lists l and integers n with n = [[length]](l),
    [[elems]](l) = { l(i) | i = 1, ..., n }
 "

 :note
 "For any given S, LRM(S) has several models which differ not
  only on how they interpret the base sorts of S other than Int and Bool,
  but also on how they interpret the 'nth' function symbol when the 
  index argument is negative or beyond the length of the list argument.
  (Similarly, for head, tail, and apply.)
 "
 )

