(theory ListWithLength

 :written_by "Philipp Ruemmer, Cesare Tinelli"
 :date "10/07/2009"

 :note "Formalization developed in cooperation with  
        Daniel Kroening, Georg Weissenbacher, and Ofer Strichman"

 :sorts ((List 1) (Int 0))

 :funs ((par (X) (nil (List X)))
        (par (X) (cons X (List X) (List X) :right_associative))
        (par (X) (head (List X) X))
        (par (X) (tail (List X) (List X)))
        (par (X) (append (List X) (List X) (List X) :left_associative))
	)

 :funs ((par (X) (length (List X) Int))
        (par (X) (nth Int (List X) X))
        )

 :funs ((NUMERAL Int) 
        (- Int Int)                       ; unary minus
        (- Int Int Int :left_associative) ; binary minus
        (+ Int Int Int :left_associative) 
        (* Int Int Int :left_associative)
        (<= Int Int Bool :chainable)
        (< Int Int Bool :chainable)
        (>= Int IntBool :chainable)
        (> Int Int Bool :chainable)
        )


 :definition 
 "Let Q be a set of sort symbols including List, Int and Bool and let
  S be the set of all (ground) sort terms over Q. For any s in S and
  any function symbol f, let [[s]] and [[f]] respectively denote the
  interpretation of s and f in some given structure.

  For any S like the above, ListsWithLength(S) is the theory consisting
  of all structures satisfying the following restrictions for all s in S:

  - [[Int]] is the set of the set Z of integer numbers.

  - [[f]] is as expected if f is in {-,+,*,<=,<,>,>=}.

  - [[n]] is as expected if n is a numeral.

  - [[(List s)]] is the set of all partial functions in the function 
    space Z -> [[s]] which are defined over the range [1 .. n] for 
    some n >= 0 and undefined elsewhere.

  - [[nil] is the (unique) function from Z to [[s]] which is undefined
    everywhere (i.e., the empty set). 

  - For all integers i and lists l, 

      [[nth]](i, l) = l(i)  if l is defined for i (and is some value of [[s]] otherwise)

  - For all lists l, 

      [[head]](l) = [[nth]](1,l)  if l is defined for 1

  - For all lists l, 

                      n  if l is defined exactly over [1 .. n], n >= 1
      [[length]](l) = 
                      0  if l is undefined everywhere

  - For all lists l, and integers i, n  with n = [[length]](l),

                       l(i + 1)     if 1 <= i <= n-1 
      [[tail]](l)(i) =
                       undefined    otherwise

  - For all lists l1, l2 and integers n1, n2 
    with n1 = [[length]](l1) and n2 = [[length]](l2),

                              l1(i)        if 1 <= i <= n1
      [[append]](l1,l2)(i) =  l1(i - n1)   if n1+1 <= i <= n1+n2
                              undefined    otherwise

  - For all x in [[s]], lists l, and integers i, n with n = [[length]](l),

                         x          if i = 1
      [[cons]](x,l)(i) = l(i - 1)   if 2 <= i <= n+1
                         undefined  otherwise
 " 

 :note 
 "For any given S, ListsWithLength(S) has several models which differ not
  only on how they interpret the base sorts of S other than Int and Bool,
  but also on how they interpret the 'nth' function symbol when the 
  index argument is negative or beyond the length of the list argument.
  (Similarly, for head and tail.)
  "
 )

